@@ -4,10 +4,12 @@ import numpy as np
 from numpy.testing import assert_allclose
 
 import astropy.units as u
-from astropy.time import Time
+from astropy.time import Time, TimeDelta
 from astropy import coordinates as coord
 from astropy.tests.helper import assert_quantity_allclose
+from astropy.modeling import models as m
 
+from .. import WCS
 from .. import coordinate_frames as cf
 
 import astropy
@@ -86,6 +88,30 @@ def test_coordinates_composite(inputs):
     assert_allclose(result[1].value, inputs[2])
 
 
+def test_coordinates_composite_order():
+    time = cf.TemporalFrame(Time("2011-01-01T00:00:00"), name='time', unit=[u.s, ], axes_order=(0, ))
+    dist = cf.CoordinateFrame(name='distance', naxes=1,
+                              axes_type=["SPATIAL"], unit=[u.m, ], axes_order=(1, ))
+    frame = cf.CompositeFrame([time, dist])
+    result = frame.coordinates(0, 0)
+    assert result[0] == Time("2011-01-01T00:00:00")
+    assert u.allclose(result[1], 0*u.m)
+
+
+def test_bare_baseframe():
+    # This is a regression test for the following call:
+    frame = cf.CoordinateFrame(1, "SPATIAL", (0,), unit=(u.km,))
+    assert u.allclose(frame.coordinate_to_quantity((1*u.m,)), 1*u.m)
+
+    # Now also setup the same situation through the whole call stack to be safe.
+    w = WCS(forward_transform=m.Tabular1D(points=np.arange(10)*u.pix,
+                                          lookup_table=np.arange(10)*u.km),
+            output_frame=frame,
+            input_frame=cf.CoordinateFrame(1, "PIXEL", (0,), unit=(u.pix,), name="detector_frame")
+            )
+    assert u.allclose(w.world_to_pixel(0*u.km), 0)
+
+
 @pytest.mark.parametrize(('frame'), coord_frames)
 def test_celestial_attributes_length(frame):
     """
@@ -138,6 +164,10 @@ def test_base_coordinate():
     assert_quantity_allclose(q1, 12 * u.deg)
     assert_quantity_allclose(q2, 3 * u.arcsec)
 
+    q1, q2 = frame.coordinate_to_quantity((12 * u.deg, 3 * u.arcsec))
+    assert_quantity_allclose(q1, 12 * u.deg)
+    assert_quantity_allclose(q2, 3 * u.arcsec)
+
 
 def test_temporal_relative():
     t = cf.TemporalFrame(reference_frame=Time("2018-01-01T00:00:00"), unit=u.s)
@@ -150,6 +180,7 @@ def test_temporal_relative():
 
     t = cf.TemporalFrame(reference_frame=Time("2018-01-01T00:00:00"))
     assert t.coordinates(10 * u.s) == Time("2018-01-01T00:00:00") + 10 * u.s
+    assert t.coordinates(TimeDelta(10, format='sec')) == Time("2018-01-01T00:00:00") + 10 * u.s
 
     a = t.coordinates((10, 20) * u.s)
     assert a[0] == Time("2018-01-01T00:00:00") + 10 * u.s
