@@ -25,11 +25,12 @@ def test_wrong_projcode2():
         gwutils.create_projection_transform("TAM")
 
 
+@pytest.mark.remote_data
 def test_fits_transform():
     hdr = fits.Header.fromfile(get_pkg_data_filename('data/simple_wcs2.hdr'))
     gw1 = gwutils.make_fitswcs_transform(hdr)
     w1 = fitswcs.WCS(hdr)
-    assert_allclose(gw1(1, 2), w1.wcs_pix2world(1, 2, 1), atol=10 ** -8)
+    assert_allclose(gw1(1, 2), w1.wcs_pix2world(1, 2, 0), atol=10 ** -8)
 
 
 def test_lon_pole():
@@ -60,12 +61,10 @@ def test_unknown_ctype():
     transform = gwutils.make_fitswcs_transform(wcsinfo)
     x = np.linspace(-5, 7, 10)
     y = np.linspace(-5, 7, 10)
-    expected = (np.array([-0.00079444, -0.0007463, -0.00069815, -0.00065, -0.00060185,
-                          -0.0005537, -0.00050556, -0.00045741, -0.00040926, -0.00036111]
-                         ),
-                np.array([-0.00075833, -0.00071019, -0.00066204, -0.00061389, -0.00056574,
-                          -0.00051759, -0.00046944, -0.0004213, -0.00037315, -0.000325]
-                         )
+    expected = (np.array([-0.00075833, -0.00071019, -0.00066204, -0.00061389, -0.00056574,
+                          -0.00051759, -0.00046944, -0.0004213 , -0.00037315, -0.000325]),
+                np.array([-0.00072222, -0.00067407, -0.00062593, -0.00057778, -0.00052963,
+                           -0.00048148, -0.00043333, -0.00038519, -0.00033704, -0.00028889])
                 )
     a, b = transform(x, y)
     assert_allclose(a, expected[0], atol=10**-8)
