@@ -11,6 +11,7 @@ from astropy.modeling import core, projections
 from astropy.io import fits
 from astropy import coordinates as coords
 from astropy import units as u
+from astropy.time import Time, TimeDelta
 
 
 # these ctype values do not include yzLN and yzLT pairs
@@ -461,6 +462,8 @@ def isnumerical(val):
         isnum = False
     elif isinstance(val, u.Quantity):
         isnum = False
+    elif isinstance(val, (Time, TimeDelta)):
+        isnum = False
     elif (isinstance(val, np.ndarray)
           and not np.issubdtype(val.dtype, np.floating)
           and not np.issubdtype(val.dtype, np.integer)):
